class MoveElement {
   protected PVector pos;
   protected PVector vel;
   protected int mass;
   protected int r;

   public void display() {
       ellipse(width - this.pos.x, height - this.pos.y - this.r, this.r * 2, this.r * 2);
   }
}

class Player extends MoveElement {
   private float leftBoundry;
   private float rightBoundry;
   
   public int points = 0;
   public int numTouch = 0;

   private int direction;

   private final int maxSpeed = 10;

   Player (float x, float y, int mass, int r, float left, float right) {
       this.pos = new PVector(x, y);
       this.vel = new PVector(0, 0);
       this.mass = mass;
       this.r = r;
       this.leftBoundry = left;
       this.rightBoundry = right;
   }

   public void move(float xVel) {
       this.vel.x += xVel;

       if (this.vel.x > this.maxSpeed) {
           this.vel.x = this.maxSpeed;
       }

       else if (this.vel.x < -this.maxSpeed) {
           this.vel.x = -this.maxSpeed;
       }
   }

   public void jump() {
       if (this.pos.y == 0) {
           this.vel.y = 25;
       }
   }
   
   public void update() {
       this.vel.x *= 0.9;
       this.pos.add(this.vel);

       if (this.pos.x - this.r < this.leftBoundry) {
           this.pos.x = this.r + this.leftBoundry;
           this.vel.x *= -0.2;
       }

       else if (this.pos.x + this.r > this.rightBoundry) {
           this.pos.x = this.rightBoundry - this.r;
           this.vel.x *= -0.2;
       }

       this.vel.y -= 10 * mass * 0.01;

       if (this.pos.y <= 0 && this.vel.y < 0) {
           this.vel.y = 0;
           this.pos.y = 0;
       }
   }
   
   public void display() {
       fill(0, 192, 255);
       super.display();
   }
   
   public void lost() {
       this.points--;
   }
}

class Ball extends MoveElement {
    private Player lastTouch;
    Ball (float x, float y) {
       this.pos = new PVector(x, y);
       this.vel = new PVector(0, 0);
       this.mass = 1;
       this.r = 30;
   }
   
   public void hitTest() {
       for (Player player : players) {
           PVector pb = new PVector(this.pos.x - player.pos.x, this.pos.y - player.pos.y);
         
           double dist = pb.mag();
       
           if (dist < this.r + player.r) {
               double u = Math.PI / 2 - Math.atan(pb.y / pb.x);
               double v = -Math.atan(this.vel.y / this.vel.x);
               
               double new_a = u + v;
               
               double tot_vel = this.vel.mag();
               
               this.vel.x = -(float)(Math.cos(new_a) * tot_vel);
               this.vel.y = (float)(Math.sin(new_a) * tot_vel);
               
               this.pos.x = player.pos.x + pb.x / pb.mag() * (this.r + player.r);
               this.pos.y = player.pos.y + pb.y / pb.mag() * (this.r + player.r);
               
               if (lastTouch != player) {
                   lastTouch = player;
                   
                   for (Player p : players) {
                       p.numTouch = 0;
                   }
               }
               
               player.numTouch++;
               
               if (player.numTouch > 3) {
                   player.lost();
               }
           }
       }
   }
   
   public void update() {
       this.hitTest();
       this.pos.add(this.vel);

       if (this.pos.x - this.r < 0) {
           this.pos.x = this.r;
           this.vel.x *= -0.8;
       }

       else if (this.pos.x + this.r > width) {
           this.pos.x = width - this.r;
           this.vel.x *= -0.8;
       }
       
       else if (this.pos.x + this.r > 390 && this.pos.x - this.r < 410) {
           if (this.pos.y < 300) {
               this.vel.x *= -1;
           }
       }

       this.vel.y -= 10 * mass * 0.01;

       if (this.pos.y <= 0) {
           this.vel.y = 0;
           this.pos.y = 0;
           
           if (this.pos.x + this.r < 390) {
               players[0].lost();
               System.out.println("Player 1 lost!  Player 1: " + players[0].points + " vs Player 2: " + players[1].points);
           }
           
           else if (this.pos.x + this.r > 410) {
               players[1].lost();
               System.out.println("Player 2 lost!  Player 1: " + players[0].points + " vs Player 2: " + players[1].points);
           }
           
           reset();
       }
   }
   
   public void display() {
       fill(200, 0, 255);
       super.display();
   }
}

Player[] players;
Ball ball;

void reset() {
    ball = new Ball(200 + (Math.random() < 0.5 ? 0 : 400), 500);
}

void setup () {
   size(800, 600);
   players = new Player[2];
   reset();

   for (int i = 0; i < 2; i++) {
       players[i] = new Player(200 + i * 400, 0, 10, 50, 410 * i, 390 + 410 * i);
   }
}

void draw () {
   fill(0, 128, 255);
   rect(0, 0, width, height);
   
   fill(0, 0, 0);
   rect(390, 300, 20, 300);

   for (Player player : players) {
       player.move(player.direction);
       player.update();
       player.display();
   }
   
   ball.update();
   ball.display();
}

void keyPressed() {
   if (keyCode == RIGHT) {
       players[0].direction--;
   }

   else if (keyCode == LEFT) {
       players[0].direction++;
   }

   else if (keyCode == UP) {
       players[0].jump();
   }
   
   if (players[0].direction < -1) players[0].direction = -1;
   if (players[0].direction > 1) players[0].direction = 1;

   if (keyCode == 68) {
       players[1].direction--;
   }

   else if (keyCode == 65) {
       players[1].direction++;
   }

   else if (keyCode == 87) {
       players[1].jump();
   }
   
   if (players[1].direction < -1) players[1].direction = -1;
   if (players[1].direction > 1) players[1].direction = 1;
}

void keyReleased() {
   if (keyCode == RIGHT) {
       players[0].direction++;
   }

   else if (keyCode == LEFT) {
       players[0].direction--;
   }

   if (keyCode == 68) {
       players[1].direction++;
   }

   else if (keyCode == 65) {
       players[1].direction--;
   }
}
